#!/bin/bash
#
# enscript.sh:	Simple workaround for getting enscript handling
#		UTF-8 partly.  Partly means that iconv is
#		used to get the UTF-8 encoding into the
#		natural laint encoding of the base language
#		provided by the enviroment variable LANG.
#
# Author: Werner Fink <werner@suse.de>
#

if test "${LANG#*.}" != "UTF-8" ; then
    exec -a enscript enscript.bin ${1+"$@"}
fi

ENC=$(LANG=${LANG%.*}; locale charmap 2> /dev/null)
test "$ENC" = "ISO-8859-1"  && ENC=ISO-8859-15
test "${LANG%.*}" = "en_US" && ENC=ISO-8859-1

case "${ENC%-*}" in
    ISO-8859|KOI8) ;;
    *) exec -a enscript enscript.bin -X $ENC ${1+"$@"}
esac

#
# All long options of enscript
#
LONG="columns:,pages:,file-align:,header:,no-header,truncate-lines,line-numbers::,\
setpagedevice:,escapes::,highlight::,font:,header-font:,print-anyway,fancy-header::,\
no-job-header,highlight-bars:,indent:,filter:,borders,page-prefeed,no-page-prefeed,\
lineprinter,lines-per-page:,mail,media:,copies:,newline:,missing-characters,output:,\
printer:,quiet,silent,landscape,portrait,baselineskip:,statusdict:,title:,tabsize:,\
underlay::,nup:,verbose,version,language:,options:,encoding:,no-formfeed,pass-through,\
color::,continuous-page-numbers,download-font:,extended-return-values,filter-stdin:,\
footer:,h-column-height:,help,help-highlight,highlight-bar-gray:,list-media,margins:,\
non-printable-format:,nup-columnwise,nup-xpad:,nup-ypad:,page-label-format:,ps-level:,\
printer-options:,rotate-even-pages,slice:,style:,swap-even-page-margins,toc,ul-angle:,\
ul-font:,ul-gray:,ul-position:,ul-style:,word-wrap"

#
# All normal options of enscript
#
SHORT="#:,1,2,a:,A:,b:,B,c,C::,d,D:,e::,E::,f:,F:,g,G,h,H:,i:,I:,j,J:,k,K,l,L:,m,M:,\
o:,O,p:,P:,q,r,R,s:,S:,t:,T:,u::,U:,v,V,w:,W:,X:,z,Z"

#
# We need the file names provided on the command line
# or the information if we read from stdin.
#
CMDLINE=$(getopt -o $SHORT -l $LONG -s bash -q -- "$@")
if test $? -ne 0 ; then
   # Let enscript do the error message
   exec -a enscript enscript.bin ${1+"$@"}
fi

#
# Why sed? Just to get the `=' back instead of ` ' the
# empty space which are inserted by getopt(1)
#
CMDLINE=$(echo "$CMDLINE" | LC_ALL=POSIX sed "s|--\([a-z-]\+\) \?\([^-]\)|--\1=\2|g")

#
# Just for encoding given on command line:
# allow the user to overwrite autodetection
#
case "${CMDLINE}" in
   *-X\ \'UTF-8\'*|*--encoding=\'UTF-8\'*)
	;; # handle this below
   *-X*|*--encoding=*|*--version*|*--help*|*-V*|*--list-media*)
	exec -a enscript enscript.bin ${1+"$@"}
esac

#
# To not fool the bash with eval here
#
  FILES=$(echo "$CMDLINE" | LC_ALL=POSIX sed  "s|.* -- \?\(.*\)|\1|")
CMDLINE=$(echo "$CMDLINE" | LC_ALL=POSIX sed  "s|\(.*\) -- \?.*|\1|")

if test "$FILES" = "'-'" ; then
    FILES=""
    eval set -- "$CMDLINE"
fi

if test -n "$FILES" ; then
    #
    # We have real files, maybe with spaces in their path name
    #
    eval set -- "$CMDLINE"
    eval fa=("$FILES")
    for f in "${fa[@]}" ; do
	title=""
	case "${CMDLINE}" in
	    *-t*|*-J*|*--title=*) ;;
	    *)	title="--title=$f"
	esac
	case "$(file -b "$f" 2> /dev/null)" in
	    ISO-8859*text)
		iconv -c -f ISO-8859-1 -t $ENC < "$f" | enscript.bin -X $ENC "$title" ${1+"$@"}
		stat=$?
		;;
	    UTF-8*text)
		iconv -c -f UTF-8      -t $ENC < "$f" | enscript.bin -X $ENC "$title" ${1+"$@"}
		stat=$?
		;;
	    ASCII*text|*)		
		enscript.bin -X $ENC ${1+"$@"} "$f"
		stat=$?
		;;
	esac
	test $stat -eq 0 || exit $stat
    done
    exit 0
fi

#
# Just handle stdin at last but not least
#
cat - | iconv -c -f UTF-8 -t $ENC | enscript.bin -X $ENC ${1+"$@"}
exit $?
